@@ -47,8 +47,13 @@ struct swaylock_args {
 	struct swaylock_colors colors;
 	enum background_mode mode;
 	char *font;
+	uint32_t font_size;
 	uint32_t radius;
 	uint32_t thickness;
+	uint32_t indicator_x_position;
+	uint32_t indicator_y_position;
+	bool override_indicator_x_position;
+	bool override_indicator_y_position;
 	bool ignore_empty;
 	bool show_indicator;
 	bool show_caps_lock_text;
@@ -57,6 +62,7 @@ struct swaylock_args {
 	bool hide_keyboard_layout;
 	bool show_failed_attempts;
 	bool daemonize;
+	bool indicator_idle_visible;
 };
 
 struct swaylock_password {
@@ -100,6 +106,7 @@ struct swaylock_surface {
 	struct pool_buffer *current_buffer;
 	bool frame_pending, dirty;
 	uint32_t width, height;
+	uint32_t indicator_width, indicator_height;
 	int32_t scale;
 	enum wl_output_subpixel subpixel;
 	char *output_name;
