@@ -74,15 +74,36 @@ void render_frame(struct swaylock_surface *surface) {
 	int arc_radius = state->args.radius * surface->scale;
 	int arc_thickness = state->args.thickness * surface->scale;
 	int buffer_diameter = (arc_radius + arc_thickness) * 2;
-	int buffer_height = buffer_diameter * 2;
 
-	int indicator_radius = state->args.radius + state->args.thickness;
-	int subsurf_xpos = surface->width / 2 - indicator_radius;
-	int subsurf_ypos = surface->height / 2 - indicator_radius;
+	int buffer_width = surface->indicator_width;
+	int buffer_height = surface->indicator_height;
+	int new_width = buffer_diameter;
+	int new_height = buffer_diameter;
+
+	int subsurf_xpos;
+	int subsurf_ypos;
+
+	// Center the indicator unless overridden by the user
+	if (state->args.override_indicator_x_position) {
+		subsurf_xpos = state->args.indicator_x_position -
+			buffer_width / (2 * surface->scale) + 2 / surface->scale;
+	} else {
+		subsurf_xpos = surface->width / 2 -
+			buffer_width / (2 * surface->scale) + 2 / surface->scale;
+	}
+
+	if (state->args.override_indicator_y_position) {
+		subsurf_ypos = state->args.indicator_y_position -
+			(state->args.radius + state->args.thickness);
+	} else {
+		subsurf_ypos = surface->height / 2 -
+			(state->args.radius + state->args.thickness);
+	}
+
 	wl_subsurface_set_position(surface->subsurface, subsurf_xpos, subsurf_ypos);
 
 	surface->current_buffer = get_next_buffer(state->shm,
-			surface->indicator_buffers, buffer_diameter, buffer_height);
+			surface->indicator_buffers, buffer_width, buffer_height);
 	if (surface->current_buffer == NULL) {
 		return;
 	}
@@ -111,10 +132,11 @@ void render_frame(struct swaylock_surface *surface) {
 	float type_indicator_border_thickness =
 		TYPE_INDICATOR_BORDER_THICKNESS * surface->scale;
 
-	if (state->args.show_indicator && state->auth_state != AUTH_STATE_IDLE) {
+	if (state->args.show_indicator && (state->auth_state != AUTH_STATE_IDLE ||
+			state->args.indicator_idle_visible)) {
 		// Draw circle
 		cairo_set_line_width(cairo, arc_thickness);
-		cairo_arc(cairo, buffer_diameter / 2, buffer_diameter / 2, arc_radius,
+		cairo_arc(cairo, buffer_width / 2, buffer_diameter / 2, arc_radius,
 				0, 2 * M_PI);
 		set_color_for_state(cairo, state, &state->args.colors.inside);
 		cairo_fill_preserve(cairo);
@@ -128,7 +150,11 @@ void render_frame(struct swaylock_surface *surface) {
 		set_color_for_state(cairo, state, &state->args.colors.text);
 		cairo_select_font_face(cairo, state->args.font,
 				CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
-		cairo_set_font_size(cairo, arc_radius / 3.0f);
+		if (state->args.font_size > 0) {
+			cairo_set_font_size(cairo, state->args.font_size);
+		} else {
+			cairo_set_font_size(cairo, arc_radius / 3.0f);
+		}
 		switch (state->auth_state) {
 		case AUTH_STATE_VALIDATING:
 			text = "verifying";
@@ -156,7 +182,7 @@ void render_frame(struct swaylock_surface *surface) {
 			}
 
 			xkb_layout_index_t num_layout = xkb_keymap_num_layouts(state->xkb.keymap);
-			if (!state->args.hide_keyboard_layout && 
+			if (!state->args.hide_keyboard_layout &&
 					(state->args.show_keyboard_layout || num_layout > 1)) {
 				xkb_layout_index_t curr_layout = 0;
 
@@ -180,7 +206,7 @@ void render_frame(struct swaylock_surface *surface) {
 			double x, y;
 			cairo_text_extents(cairo, text, &extents);
 			cairo_font_extents(cairo, &fe);
-			x = (buffer_diameter / 2) -
+			x = (buffer_width / 2) -
 				(extents.width / 2 + extents.x_bearing);
 			y = (buffer_diameter / 2) +
 				(fe.height / 2 - fe.descent);
@@ -189,6 +215,10 @@ void render_frame(struct swaylock_surface *surface) {
 			cairo_show_text(cairo, text);
 			cairo_close_path(cairo);
 			cairo_new_sub_path(cairo);
+
+			if (new_width < extents.width) {
+				new_width = extents.width;
+			}
 		}
 
 		// Typing indicator: Highlight random part on keypress
@@ -197,7 +227,7 @@ void render_frame(struct swaylock_surface *surface) {
 			static double highlight_start = 0;
 			highlight_start +=
 				(rand() % (int)(M_PI * 100)) / 100.0 + M_PI * 0.5;
-			cairo_arc(cairo, buffer_diameter / 2, buffer_diameter / 2,
+			cairo_arc(cairo, buffer_width / 2, buffer_diameter / 2,
 					arc_radius, highlight_start,
 					highlight_start + TYPE_INDICATOR_RANGE);
 			if (state->auth_state == AUTH_STATE_INPUT) {
@@ -217,12 +247,12 @@ void render_frame(struct swaylock_surface *surface) {
 
 			// Draw borders
 			cairo_set_source_u32(cairo, state->args.colors.separator);
-			cairo_arc(cairo, buffer_diameter / 2, buffer_diameter / 2,
+			cairo_arc(cairo, buffer_width / 2, buffer_diameter / 2,
 					arc_radius, highlight_start,
 					highlight_start + type_indicator_border_thickness);
 			cairo_stroke(cairo);
 
-			cairo_arc(cairo, buffer_diameter / 2, buffer_diameter / 2,
+			cairo_arc(cairo, buffer_width / 2, buffer_diameter / 2,
 					arc_radius, highlight_start + TYPE_INDICATOR_RANGE,
 					highlight_start + TYPE_INDICATOR_RANGE +
 						type_indicator_border_thickness);
@@ -232,10 +262,10 @@ void render_frame(struct swaylock_surface *surface) {
 		// Draw inner + outer border of the circle
 		set_color_for_state(cairo, state, &state->args.colors.line);
 		cairo_set_line_width(cairo, 2.0 * surface->scale);
-		cairo_arc(cairo, buffer_diameter / 2, buffer_diameter / 2,
+		cairo_arc(cairo, buffer_width / 2, buffer_diameter / 2,
 				arc_radius - arc_thickness / 2, 0, 2 * M_PI);
 		cairo_stroke(cairo);
-		cairo_arc(cairo, buffer_diameter / 2, buffer_diameter / 2,
+		cairo_arc(cairo, buffer_width / 2, buffer_diameter / 2,
 				arc_radius + arc_thickness / 2, 0, 2 * M_PI);
 		cairo_stroke(cairo);
 
@@ -248,9 +278,8 @@ void render_frame(struct swaylock_surface *surface) {
 			cairo_text_extents(cairo, layout_text, &extents);
 			cairo_font_extents(cairo, &fe);
 			// upper left coordinates for box
-			x = (buffer_diameter / 2) - (extents.width / 2) - box_padding;
-			y = (buffer_diameter / 2) + arc_radius + arc_thickness/2 +
-				box_padding; // use box_padding also as gap to indicator
+			x = (buffer_width / 2) - (extents.width / 2) - box_padding;
+			y = buffer_diameter;
 
 			// background box
 			cairo_rectangle(cairo, x, y,
@@ -261,7 +290,6 @@ void render_frame(struct swaylock_surface *surface) {
 			// border
 			cairo_set_source_u32(cairo, state->args.colors.layout_border);
 			cairo_stroke(cairo);
-			cairo_new_sub_path(cairo);
 
 			// take font extents and padding into account
 			cairo_move_to(cairo,
@@ -270,6 +298,18 @@ void render_frame(struct swaylock_surface *surface) {
 			cairo_set_source_u32(cairo, state->args.colors.layout_text);
 			cairo_show_text(cairo, layout_text);
 			cairo_new_sub_path(cairo);
+
+			new_height += fe.height + 2 * box_padding;
+			if (new_width < extents.width + 2 * box_padding) {
+				new_width = extents.width + 2 * box_padding;
+			}
+		}
+
+		if (buffer_width != new_width || buffer_height != new_height) {
+			destroy_buffer(surface->current_buffer);
+			surface->indicator_width = new_width;
+			surface->indicator_height = new_height;
+			render_frame(surface);
 		}
 	}
 
